@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Copyright (C) 2006 - Steve Frécinaux
-#               2016 - Nick Boultbee
+#            2016-17 - Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,9 +26,11 @@ import traceback
 
 from gi.repository import Gtk, Pango, Gdk, GLib
 
-from quodlibet import _
+from quodlibet import _, app, ngettext
 from quodlibet import const
-from quodlibet.qltk import Icons
+from quodlibet.plugins.events import EventPlugin
+from quodlibet.plugins.gui import UserInterfacePlugin
+from quodlibet.qltk import Icons, add_css, Align
 from quodlibet.compat import exec_, PY2
 from quodlibet.plugins.songsmenu import SongsMenuPlugin
 from quodlibet.util.collection import Collection
@@ -38,58 +40,83 @@ from quodlibet.util import print_
 class PyConsole(SongsMenuPlugin):
     PLUGIN_ID = 'Python Console'
     PLUGIN_NAME = _('Python Console')
-    PLUGIN_DESC = _('Interactive Python console.')
+    PLUGIN_DESC = _('Interactive Python console. Opens a new window.')
     PLUGIN_ICON = Icons.UTILITIES_TERMINAL
 
     def plugin_songs(self, songs):
-        win = ConsoleWindow(songs)
+        desc = ngettext("%d song", "%d songs", len(songs)) % len(songs)
+        win = ConsoleWindow(create_console(songs), title=desc)
         win.set_icon_name(self.PLUGIN_ICON)
-        win.set_title(self.PLUGIN_DESC + " (Quod Libet)")
+        win.set_title(_("{plugin_name} for {songs} ({app})").format(
+            plugin_name=self.PLUGIN_NAME, songs=desc, app=app.name))
         win.show_all()
 
 
+class PyConsoleSidebar(EventPlugin, UserInterfacePlugin):
+    PLUGIN_ID = 'Python Console Sidebar'
+    PLUGIN_NAME = _('Python Console Sidebar')
+    PLUGIN_DESC = _('Interactive Python console sidebar, '
+                    'that follows the selected songs in the main window.')
+    PLUGIN_ICON = Icons.UTILITIES_TERMINAL
+
+    def enabled(self):
+        self.console = create_console()
+
+    def plugin_on_songs_selected(self, songs):
+        self.console.namespace = namespace_for(songs)
+
+    def create_sidebar(self):
+        align = Align(self.console)
+        self.sidebar = align
+        self.sidebar.show_all()
+        return align
+
+
+def create_console(songs=None):
+    console = PythonConsole(namespace_for(songs)) if songs else PythonConsole()
+    access_string = _("You can access the following objects by default:")
+    access_string += "\\n".join([
+                    "",
+                    "  %5s: SongWrapper objects",
+                    "  %5s: Song dictionaries",
+                    "  %5s: Filename list",
+                    "  %5s: Songs Collection",
+                    "  %5s: Application instance"]) % (
+                       "songs", "sdict", "files", "col", "app")
+
+    dir_string = _("Your current working directory is:")
+
+    if PY2:
+        console.eval("from __future__ import print_function", False)
+    console.eval("import mutagen", False)
+    console.eval("import os", False)
+    console.eval("print(\"Python: %s / Quod Libet: %s\")" %
+                 (sys.version.split()[0], const.VERSION), False)
+    console.eval("print(\"%s\")" % access_string, False)
+    console.eval("print(\"%s \"+ os.getcwd())" % dir_string, False)
+    return console
+
+
+def namespace_for(song_wrappers):
+    files = [song('~filename') for song in song_wrappers]
+    song_dicts = [song._song for song in song_wrappers]
+    collection = Collection()
+    collection.songs = song_dicts
+    return {
+        'songs': song_wrappers,
+        'files': files,
+        'sdict': song_dicts,
+        'col': collection,
+        'app': app}
+
+
 class ConsoleWindow(Gtk.Window):
-    def __init__(self, songs):
+    def __init__(self, console, title=None):
         Gtk.Window.__init__(self)
-
-        files = [song('~filename') for song in songs]
-        song_dicts = [song._song for song in songs]
-        collection = Collection()
-        collection.songs = song_dicts
-
-        self.set_size_request(700, 500)
-
-        from quodlibet import app
-        console = PythonConsole(
-            namespace={
-                'songs': songs,
-                'files': files,
-                'sdict': song_dicts,
-                'col': collection,
-                'app': app})
+        if title:
+            self.set_title(title)
         self.add(console)
-
-        access_string = _("You can access the following objects by default:")
-        access_string += "\\n".join([
-                        "",
-                        "  %5s: SongWrapper objects",
-                        "  %5s: Song dictionaries",
-                        "  %5s: Filename list",
-                        "  %5s: Songs Collection",
-                        "  %5s: Application instance"]) % (
-                           "songs", "sdict", "files", "col", "app")
-
-        dir_string = _("Your current working directory is:")
-
-        if PY2:
-            console.eval("from __future__ import print_function", False)
-        console.eval("import mutagen", False)
-        console.eval("import os", False)
-        console.eval("print(\"Python: %s / Quod Libet: %s\")" %
-                     (sys.version.split()[0], const.VERSION), False)
-        console.eval("print(\"%s\")" % access_string, False)
-        console.eval("print(\"%s \"+ os.getcwd())" % dir_string, False)
-
+        self.set_size_request(700, 500)
         console.connect("destroy", lambda *x: self.destroy())
 
 
@@ -99,8 +126,9 @@ class PythonConsole(Gtk.ScrolledWindow):
 
         self.destroy_cb = destroy_cb
         self.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
-        self.set_shadow_type(Gtk.ShadowType.IN)
+        self.set_shadow_type(Gtk.ShadowType.NONE)
         self.view = Gtk.TextView()
+        add_css(self, "* { background-color: white; padding: 6px; } ")
         self.view.modify_font(Pango.font_description_from_string('Monospace'))
         self.view.set_editable(True)
         self.view.set_wrap_mode(Gtk.WrapMode.CHAR)
@@ -131,8 +159,8 @@ class PythonConsole(Gtk.ScrolledWindow):
         self.namespace['__history__'] = self.history
 
         # Set up hooks for standard output.
-        self.stdout = OutFile(self, sys.stdout.fileno(), self.normal)
-        self.stderr = OutFile(self, sys.stderr.fileno(), self.error)
+        self.stdout = OutFile(self, self.normal)
+        self.stderr = OutFile(self, self.error)
 
         # Signals
         self.view.connect("key-press-event", self.__key_press_event_cb)
@@ -351,8 +379,7 @@ class OutFile(object):
     """A fake output file object. It sends output to a TK test widget,
     and if asked for a file number, returns one set on instance creation"""
 
-    def __init__(self, console, fn, tag):
-        self.fn = fn
+    def __init__(self, console, tag):
         self.console = console
         self.tag = tag
 
@@ -363,7 +390,7 @@ class OutFile(object):
         pass
 
     def fileno(self):
-        return self.fn
+        raise IOError
 
     def isatty(self):
         return 0
