@@ -130,7 +130,7 @@ class PlayerOptions(GObject.Object):
         When `repeat` is True the current song will be replayed.
         """
 
-        return (self._order_widget.repeated and
+        return (self._order_widget and self._order_widget.repeated and
                 self._order_widget.repeater is RepeatSongForever)
 
     @single.setter
@@ -348,7 +348,7 @@ class TopBar(Gtk.Toolbar):
 
         # On older Gtk+ (3.4, at least)
         # setting a margin on CoverImage leads to errors and result in the
-        # QL window not beeing visible for some reason.
+        # QL window not being visible for some reason.
         assert self.image.props.margin == 0
 
         for child in self.get_children():
@@ -470,9 +470,9 @@ class AppMenu(object):
             action.connect("activate", callback)
 
         # export on the bus
-        ag_object_path = "/net/sacredchao/QuodLibet"
-        am_object_path = "/net/sacredchao/QuodLibet/menus/appmenu"
-        app_id = "net.sacredchao.QuodLibet"
+        ag_object_path = "/io/github/quodlibet/QuodLibet"
+        am_object_path = "/io/github/quodlibet/QuodLibet/menus/appmenu"
+        app_id = "io.github.quodlibet.QuodLibet"
 
         win = window.get_window()
         if not hasattr(win, "set_utf8_property"):
